@@ -84,6 +84,8 @@ def symbolType(line, symbol, scope, member=""):
                         raise StartError(f"In line {line}, an unknown member '{elem.split('/')[1]}' is used!")
                     elif org_type_match in built_in_types:
                         raise StartError(f"In line {line}, a member is accessed of a simple type with '{elem}' which is not possible!")
+                    elif (elem in built_in_functions or elem in user_functions) and org_type_match in types:
+                        raise StartError(f"In line {line}, '{elem}' is used to get a member of {org_type_match} which is not possible as '{elem}' has the wrong return type!")
                     else:
                         raise StartError(f"In line {line}, '{elem}' is used which is undefined!")
             return type_match
@@ -116,11 +118,15 @@ def match(line, symbol, scope, member):
     functions_name = f"{member}.{symbol}" if member else symbol
     if functions_name in user_functions:
         return user_functions[functions_name]
+    elif symbol in user_functions and (not MEMBER_TYPES_RESTRICTED or user_functions[symbol] == "string") and member not in built_in_types and member in types:
+        if len(t := set(variables[member].values())) == 1:  # If a type has only members of one type we also know the potential type
+            return list(t)[0]
+        return "*"
 
     # Search in built-in functions
     if symbol in built_in_functions and member == "":  # or ignore types, symbol not in types
         return built_in_functions[symbol]
-    elif symbol in built_in_functions and (MEMBER_TYPES_RESTRICTED or built_in_functions[symbol] == "string") and member not in built_in_types and member in types:
+    elif symbol in built_in_functions and (not MEMBER_TYPES_RESTRICTED or built_in_functions[symbol] == "string") and member not in built_in_types and member in types:
         if len(t := set(variables[member].values())) == 1:  # If a type has only members of one type we also know the potential type
             return list(t)[0]
         return "*"
@@ -132,6 +138,8 @@ def match(line, symbol, scope, member):
         return variables[member][symbol]
     elif "*" in variables[member] and (not MEMBER_TYPES_RESTRICTED or symbolType(line, symbol, scope) == "number"):
         return variables[member]["*"]
+    elif "0" in variables[member] and (not MEMBER_TYPES_RESTRICTED or symbolType(line, symbol, scope) == "number"):
+        return variables[member]["0"]
 
     return None
 
@@ -154,8 +162,10 @@ def toSymbol(t, scope, line):
             return name
 
     elif t.data == "class_variable":
-        if get(t, 'variable') is not None:
+        if get(t, 'variable') is not None:  # get an attribute from a class
             return f"{scope.split('.', 1)[0]}.{toSymbol(get(t, 'variable'), scope, line)}"
+        elif get(t, 'expression') is not None:  # a variable is used to get an attribute from a class
+            return f"{scope.split('.', 1)[0]}.{symbolType(line, toSymbol(t.children[0], scope, line), scope)}"
         else:  # something is wrong
             return f"{scope.split('.', 1)[0]}.{toSymbol(t.children[0], scope, line)}"
 
@@ -236,6 +246,14 @@ def add_function(function, rtype, line):
 
     user_functions[function] = rtype
 
+def variable_exists(name, scope, line):
+    # print(name, scope, line)
+    if name == "this":
+        if len(scope.split(".")) != 2:
+            raise StartError(f"In line {line}, '{name}' is used which is outside a type which is not possible.")
+    elif name not in variables[scope]:
+        raise StartError(f"In line {line}, '{name}' is used which is not declared.")
+
 def toPython(t, state, indent, scope):
     """
     This goes through the grammar tree and recursively builds the python file
@@ -396,9 +414,6 @@ def toPython(t, state, indent, scope):
     elif t.data == "statement":  # pass it on with a newline for the next statement
         return f"{indent}{toPython(t.children[0], state, indent, scope)}\n"
 
-    elif t.data == "statement_blocks":
-        return f"{indent}{toPython(t.children[0], state, indent, scope)}"
-
     elif t.data == 'assignment':
         # Check the types of both sides of the assigment
         var = get(t, "variable") or get(t, "class_variable")
@@ -449,22 +464,28 @@ def toPython(t, state, indent, scope):
                 return f"{expression}.copy(getattr({var}))"
 
     elif t.data == 'variable':  # variable Tree, so its bracketed
-        if t.children[0].data == 'class_variable':
+        if t.children[0].data == 'class_variable':  # a member of a class variable: self.coord ...
             name = toPython(get(t, "expression"), state, "", scope)
-            if get(t.children[0], 'variable') is not None:
+            if get(t.children[0], 'variable') is not None:  # a member of a member of a class variable: self.coord["x"]
                 variable = val(get(t.children[0], 'variable'), 'name')
+                variable_exists(variable, scope.split(".")[0], line)
                 try:
-                    return f"self.{variable}, '{str(eval(name))}'"
+                    name = f"'{str(eval(name))}'"
                 except NameError:
-                    return f"self.{variable}, str({name})"
-            else:
+                    name = f"str({name})"
+                return f"self.{variable}, {name}"
+            else:  # a member of a member (expressed as variable) of a class variable: getitem(getitem(self, <variable>), name)
+                # name checking is not possible here because the object is unknown due to the variable
                 return f"{str(toPython(t.children[0], state, '', scope))}, str({name})"
         elif val(t, "name") is not None:
             if val(t, "name") == "this":
+                variable_exists("this", scope, line)
                 return f"self"
+
+            variable_exists(val(t, "name"), scope, line)
             return f"local_vars['{val(t, 'name')}']"
-        else:
-            key = toPython(get(t, "expression"), state, "", scope)
+        else:  # getitem from variable
+            key = toPython(get(t, "expression"), state, "", scope)  # key is checked in toPython
             variable = val(get(t, 'variable'), 'name')
             # this tries to make the python code a bit more readable by making a key string into a python str
             try:
@@ -473,16 +494,20 @@ def toPython(t, state, indent, scope):
                 key = f"str({key})"
 
             if variable is None:  # deeply nested attribute calls need to be handled differently
+                # Variable should be checked in the nesting
                 variable = toPython(get(t, "variable"), state, "", scope)
                 return f"getattr({variable}), {key}"
+            variable_exists(variable, scope, line)
             return f"local_vars['{variable}'], {key}"
 
-    elif t.data == "class_variable":
-        if get(t, 'variable') is None:
+    elif t.data == "class_variable":  # Using "this[..]"
+        if get(t, 'variable') is None:  # expression is used in this[<expr>]
             attr = toPython(t.children[0], state, indent, scope)
+            # TODO: test using tosymbol if attr is correct type
             return f"self.__dict__[str({attr})]"
 
         variable = val(get(t, 'variable'), 'name')
+        variable_exists(variable, scope.split(".")[0], line)
         return f"self.{variable}"
 
     elif t.data == "expression":  # just relay to the next Tree object
@@ -508,7 +533,7 @@ def toPython(t, state, indent, scope):
             return f"{variable}.{function}({arguments})"
 
         if function not in user_functions and function not in built_in_functions:
-            raise StartError(f"In line {line}: Function or constructor {function} not defined in '{scope_2_str(scope)}' scope!")
+            raise StartError(f"In line {line}: Function or constructor {function} not defined in '{scope_2_str(scope if len(scope.split('.')) < 2 else '')}' scope!")
 
         if function in built_in_functions:
             function = built_in_function_names[function]
@@ -543,8 +568,12 @@ def toPython(t, state, indent, scope):
             return val(t, 'string')
 
     elif t.data == "if_block":
+        # Make sure to use getattr if you just use a nested variable
+        statement = toPython(get(t, 'expression'), state, '', scope)
+        if get(t.children[0], 'variable') and get(t.children[0].children[0], 'variable'):
+            statement = f"getattr({statement})"
         # start an if block with an Enter
-        result = f"\n{indent}if {toPython(get(t, 'expression'), state, '', scope)}:\n"
+        result = f"\n{indent}if {statement}:\n"
         result += toPython(get(t, 'block'), state, indent + '\t', scope)
         # check for else
         if (else_block := get(t, 'else_block')) is not None:
@@ -553,8 +582,12 @@ def toPython(t, state, indent, scope):
         return result
 
     elif t.data == "while_block":
+        # Make sure to use getattr if you use a nested variable
+        statement = toPython(get(t, 'expression'), state, '', scope)
+        if get(t.children[0], 'variable') and get(t.children[0].children[0], 'variable'):
+            statement = f"getattr({statement})"
         # start a while block with an Enter
-        result = f"\n{indent}while {toPython(get(t, 'expression'), state, '', scope)}:\n"
+        result = f"\n{indent}while {statement}:\n"
         result += toPython(get(t, 'block'), state, indent + '\t', scope)
         return result
 
@@ -707,10 +740,17 @@ def main(fileName, long_names):
         source = file.read() + "\n"  # Make sure there is always a proper end of file
 
     try:
-        parser= Lark(grammar, parser='earley', propagate_positions=True)
-        parse_tree = parser.parse(source)
-    except UnexpectedInput as e:
-        raise StartError(f"Syntax Error in line {e.line}: \n{e._context}")
+        try:
+            parser = Lark(grammar, parser='earley', propagate_positions=True)
+            parse_tree = parser.parse(source)
+        except UnexpectedInput as e:
+            try:
+                raise StartError(f"Syntax Error in line {e.line}: \n{e._context}")
+            except AttributeError:
+                raise StartError(f"Syntax Error in line {e.line}. Probably, somewhere in your code an 'end' is missing.")
+    except StartError as e:
+        print(e)
+        exit()
 
     parse_tree = ParseTransform1().transform(parse_tree)
     parse_tree = ParseTransform2().transform(parse_tree)
@@ -727,7 +767,7 @@ def main(fileName, long_names):
         warnings.warn(f"In line {line}, the variable '{var}' in scope '{scope}' is defined as <{var_type}> before the type is declared. Make sure that you declare a type before using it (if possible)!")
 
     f = open((Path(fileName).parent.resolve() / Path(fileName).stem).with_suffix(".py"), "w")
-    import_string = "import import_start\n\n"
+    import_string = "from start_compiler import import_start\n\n"
     import_string += f"import_start.LONG_NAMES = {long_names}\n\n"
     import_string += "attrlist = dir(import_start)\n"
     import_string += "for attr in attrlist:\n"
